﻿using Newtonsoft.Json.Converters;
using Spread.Common.DataStore.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Spread.Common.DataStore.Utilities;
using Newtonsoft.Json;

namespace Spread.Common.DataStore
{
    public class DashBoardDataSource
    {
        public DashBoardItems LoadDashBoardItems()
        {
            DashBoardItems dashBoardItems = new DashBoardItems();
            var format = "dd/MM/yyyy";
            var dateTimeConverter = new IsoDateTimeConverter { DateTimeFormat = format };

            string jsonStr = FileUtilities.LoadJSON("Companies.json");
            dashBoardItems.Companies = JsonConvert.DeserializeObject<List<Company>>(jsonStr, dateTimeConverter);

            jsonStr = FileUtilities.LoadJSON( "DashBoardCustomers.json");
            dashBoardItems.Customers = JsonConvert.DeserializeObject<List<Customer>>(jsonStr, dateTimeConverter);

            jsonStr = FileUtilities.LoadJSON( "CustomerTarget.json");
            dashBoardItems.CustomerTargets = JsonConvert.DeserializeObject<List<CustomerTarget>>(jsonStr, dateTimeConverter);
            AssignYearToData(dashBoardItems.CustomerTargets.ToList<IDateTime>());

            jsonStr = FileUtilities.LoadJSON( "DashBoardInvoices.json");
            dashBoardItems.Invoices = JsonConvert.DeserializeObject<List<Invoice>>(jsonStr, dateTimeConverter);
            AssignDateToInvoice(dashBoardItems.Invoices);

            jsonStr = FileUtilities.LoadJSON( "MarketSales.json");
            dashBoardItems.MarketSales = JsonConvert.DeserializeObject<List<MarketSale>>(jsonStr, dateTimeConverter);
            AssignYearToData(dashBoardItems.MarketSales.ToList<IDateTime>());

            jsonStr = FileUtilities.LoadJSON( "Products.json");
            dashBoardItems.Products = JsonConvert.DeserializeObject<List<Product>>(jsonStr, dateTimeConverter);

            jsonStr = FileUtilities.LoadJSON( "Regions.json");
            dashBoardItems.Regions = JsonConvert.DeserializeObject<List<Region>>(jsonStr, dateTimeConverter);
            AssignYearToData(dashBoardItems.Regions.ToList<IDateTime>());

            jsonStr = FileUtilities.LoadJSON( "Sales.json");
            dashBoardItems.Sales = JsonConvert.DeserializeObject<List<Sale>>(jsonStr, dateTimeConverter);

            jsonStr = FileUtilities.LoadJSON( "DashBoardTargetMaster.json");
            dashBoardItems.TargetMasters = JsonConvert.DeserializeObject<List<DashBoardTargetMaster>>(jsonStr, dateTimeConverter);
            AssignYearToData(dashBoardItems.TargetMasters.ToList<IDateTime>());

            UpdateActualSaleForCompany(dashBoardItems);
            return dashBoardItems;
        }

        //Update date based on the current year or datetime
        private void AssignDateToInvoice(List<Invoice> invoiceList)
        {
            invoiceList = invoiceList.OrderBy(c => c.ID).ToList();
            int calcMonth = 1;
            int currentYr = DateTime.Now.Year;
            int currentMonth = DateTime.Now.Month;
            int count = invoiceList.Count();
            Random rand = new Random();
            foreach (Invoice item in invoiceList)
            {
                var itemID = Double.Parse(item.ID);
                double d = itemID / (count / 24);
                calcMonth = (int)Math.Ceiling(d);
                if (calcMonth <= currentMonth && itemID <= (count / 2))
                {
                    item.CreatedOn  = new DateTime(currentYr, calcMonth, rand.Next(1, 28));
                }
                else if (itemID <= (count / 2))
                {
                    item.CreatedOn = new DateTime(currentYr - 2, calcMonth, rand.Next(1, 28));
                }
                else
                {
                    item.CreatedOn = new DateTime(currentYr - 1, calcMonth - 12, rand.Next(1, 28));
                }
            }
        }
        //Update year in data based on the current datetime
        private void AssignYearToData(List<IDateTime> list)
        {
            list = list.OrderBy(c => c.Id).ToList();
            int currentYr = DateTime.Now.Year;
            int currentMonth = DateTime.Now.Month;
            int count = list.Count();
            foreach (IDateTime item in list)
            {
                if (currentMonth >= item.Month && item.Id <= (count / 2))
                {
                    item.Year = currentYr;
                }
                else if (item.Id <= (count / 2))
                {
                    item.Year = currentYr - 2;
                }
                else
                {
                    item.Year = currentYr - 1;
                }
            }
        }
        //Update actual sale for company
        private void UpdateActualSaleForCompany(DashBoardItems dashBoardItems)
        {
            foreach (MarketSale marketSale in dashBoardItems.MarketSales)
            {
                if (marketSale.CompId == 3)
                {
                    marketSale.TotalSale = (double)dashBoardItems.Invoices.Where(i => i.CreatedOn.Month 
                    == marketSale.Month && i.CreatedOn.Year == marketSale.Year).Sum(i => i.TotalAmount);
                }
            }
        }

    }
}
