﻿using System;
using System.Collections.Generic;
using System.Linq;
using Newtonsoft.Json;
using Spread.Common.DataStore.Models;
using Spread.Common.DataStore.Utilities;

namespace Spread.Common.DataStore.DataSourceEntities
{
    public class IncomeStatementDataSource
    {
        #region Public methods
        // Get a List of all BusinessStat from JSON
        public List<BusinessStat> GetAllBusinessStats()
        {
            string json = FileUtilities.LoadJSON("BusinessStats.json");
            List<BusinessStat> stats = JsonConvert.DeserializeObject<List<BusinessStat>>(json);

            UpdateYearInStats(stats);

            return stats;
        }
        #endregion

        #region Private methods
        // Update 'Year' Property of BusinessStat to keep the data current
        private void UpdateYearInStats(IEnumerable<BusinessStat> stats)
        {
            stats = stats.OrderByDescending(stat => stat.Year).ThenBy(stat => stat.Month).AsEnumerable();
            
            int diff = DateTime.Now.Year - stats.First().Year;

            foreach (var stat in stats)
            {
                stat.Year = stat.Year + diff;
            }
        }
        #endregion
    }
}
