﻿using Spread.Common.DataStore.Models;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using System;
using System.Collections.Generic;
using System.Linq;
using Spread.Common.DataStore.Utilities;

namespace Spread.Common.DataStore
{
    public class JsonDataSource
    {

        #region Fields
        private const string _jsonPath = @".\Data";
        #endregion

        #region ctor
        internal JsonDataSource()
        {
        }
        #endregion

        #region Private methods
        private string GetJSonFromFile(string filePath)
        {
            return FileUtilities.ReadTextFromFile(filePath);
        }

        private void UpdateDateInMatchData(List<Match> matchListCurrent, List<Match> matchListPrevious, string selectedYr)
        {
            int year = 0;
            int monthStart = 0;
            int dayStart = 0;
            int monthLast = 0;
            int dayLast = 0;
            Random gen = new Random();
            DateTime start = DateTime.Now;
            int range = 0;
            string[] selectedYrs = selectedYr.Split('-');
            int yearSelected = selectedYrs.Count() < 2 ? Convert.ToInt32(selectedYrs[0]) : Convert.ToInt32(selectedYrs[1]);

            if (yearSelected == DateTime.Now.Year)
            {
                year = yearSelected;
                monthStart = 1;
                dayStart = 1;
                monthLast = DateTime.Now.Month;
                dayLast = DateTime.Now.Day;
                start = new DateTime(year, monthStart, dayStart);
                range = (new DateTime(year, monthLast, dayLast) - start).Days;

            }
            else
            {
                year = yearSelected;
                monthStart = 1;
                dayStart = 1;
                monthLast = 12;
                dayLast = 31;
                start = new DateTime(year, monthStart, dayStart);
                range = (new DateTime(year, monthLast, dayLast) - start).Days;

            }
            foreach (Match match in matchListCurrent)
            {
                match.MatchDate = RandomDay(gen, range, start);
            }
            year = yearSelected - 1;
            monthStart = 1;
            dayStart = 1;
            monthLast = 12;
            dayLast = 31;
            start = new DateTime(year, monthStart, dayStart);
            range = (new DateTime(year, monthLast, dayLast) - start).Days;

            foreach (Match match in matchListPrevious)
            {
                match.MatchDate = RandomDay(gen, range, start);
            }
        }

        private DateTime RandomDay(Random gen, int range, DateTime start)
        {
            return start.AddDays(gen.Next(range));
        }
        #endregion

        #region Public methods

        /// <summary>
        /// Get a List of all Customers
        /// </summary>
        /// <returns>List of all Customers</returns>
        public List<Customer> GetAllCustomers()
        {
            string json = System.IO.File.ReadAllText(System.IO.Path.Combine(_jsonPath, "Customers.json"));
            List<Customer> customers = JsonConvert.DeserializeObject<List<Customer>>(json);
            return customers;
        }
        public List<Customer> GetAgingReportCustomers()
        {
            string json = System.IO.File.ReadAllText(System.IO.Path.Combine(_jsonPath, "AgingReportCustomers.json"));
            List<Customer> customers = JsonConvert.DeserializeObject<List<Customer>>(json);
            return customers;
        }

        public List<Parts> GetAllParts()
        {
            string json = System.IO.File.ReadAllText(System.IO.Path.Combine(_jsonPath, "Parts.json"));
            List<Parts> parts = JsonConvert.DeserializeObject<List<Parts>>(json);

            return parts;
        }
        public List<string> GetAllInvoiceIDs()
        {
            List<Invoice> invoices = GetAllInvoices();
            return invoices.Select(inv => inv.ID).ToList();
        }
        public List<Invoice> GetAllInvoices()
        {
            string json = string.Empty;
            try
            {
                json = FileUtilities.LoadJSON("Invoice.json");
            }
            catch
            {
                json = System.IO.File.ReadAllText(System.IO.Path.Combine(_jsonPath, "Invoice.json"));
            }                
            List<Invoice> invoices = JsonConvert.DeserializeObject<List<Invoice>>(json);
            return invoices;
        }
        public List<Invoice> GetAgingReportInvoices()
        {
            string json = System.IO.File.ReadAllText(System.IO.Path.Combine(_jsonPath, "AgingReportInvoices.json"));
            List<Invoice> invoices = JsonConvert.DeserializeObject<List<Invoice>>(json);

            return invoices;
        }
        public List<Invoice> GetAgingReportInvoices(DateTime startDate, DateTime endDate)
        {
            Random random = new Random();
            List<Invoice> invoices = GetAgingReportInvoices();
            invoices = invoices.Where(inv => inv.CreatedOn >= startDate && inv.CreatedOn <= endDate).
                OrderBy(inv => random.Next()).ToList();
            int count = random.Next(Math.Min(60, invoices.Count), 300);
            invoices = invoices.Take(Math.Min(count, invoices.Count)).ToList();

            // Update 'ID' property of Invoice to be increasing
            int id = 101;
            for (int i = 0; i < invoices.Count; ++i)
            {
                invoices[i].ID = id++.ToString();
            }

            return invoices.ToList();
        }

        public Customer GetCustomer(int id)
        {
            List<Customer> customers = GetAllCustomers();
            return customers.Where(cust => cust.ID == id).FirstOrDefault();
        }
        public Vehicle GetVehicle(string id)
        {
            string json = System.IO.File.ReadAllText(System.IO.Path.Combine(_jsonPath, "Vehicle.json"));
            List<Vehicle> vehicles = JsonConvert.DeserializeObject<List<Vehicle>>(json);

            return vehicles.Where(v => v.ID == id).FirstOrDefault();
        }
        public Invoice GetInvoice(string id)
        {
            List<Invoice> invoices = GetAllInvoices();
            return invoices.Where(inv => inv.ID == id).FirstOrDefault();
        }
        public Parts GetParts(string id)
        {
            List<Parts> parts = GetAllParts();
            return parts.Where(p => p.ID == id).FirstOrDefault();
        }
        public List<InvoiceParts> GetInvoiceParts(string invoiceID)
        {
            string json = System.IO.File.ReadAllText(System.IO.Path.Combine(_jsonPath, "InvoiceParts.json"));
            List<InvoiceParts> invoiceParts = JsonConvert.DeserializeObject<List<InvoiceParts>>(json);
            return invoiceParts.Where(invP => invP.InvoiceID == invoiceID).ToList();
        }
        public List<InvoiceServices> GetInvoiceServices(string invoiceID)
        {
            string json = System.IO.File.ReadAllText(System.IO.Path.Combine(_jsonPath, "InvoiceServices.json"));
            List<InvoiceServices> invoiceServices = JsonConvert.DeserializeObject<List<InvoiceServices>>(json);

            return invoiceServices.Where(invS => invS.InvoiceID == invoiceID).ToList();
        }
        public void UpdateInvoice(Invoice invoice)
        {
            List<Invoice> invoices = GetAllInvoices();
            Invoice newInvoice = invoices.Where(inv => inv.ID == invoice.ID).FirstOrDefault();
            newInvoice.TotalPartsAmount = invoice.TotalPartsAmount;
            newInvoice.TotalServiceAmount = invoice.TotalServiceAmount;
            newInvoice.TotalAmount = invoice.TotalAmount;
            try
            {
              System.IO.File.WriteAllText(System.IO.Path.Combine(_jsonPath, "Invoice.json"), JsonConvert.SerializeObject(invoices, Formatting.Indented));
            }
            catch { }
        }
        public List<Invoice> GetInvoiceByCustomerID(int customerID)
        {
            List<Invoice> invoices = GetAllInvoices();
            return invoices.Where(inv => inv.CustomerID == customerID).ToList();
        }
        public List<Invoice> GetInvoiceByVehicleID(string vehicleID)
        {
            List<Invoice> invoices = GetAllInvoices();
            return invoices.Where(inv => inv.VehicleID == vehicleID).ToList();
        }
        public List<Invoice> GetInvoiceByCustomerName(string customerName)
        {
            string json = System.IO.File.ReadAllText(System.IO.Path.Combine(_jsonPath, "Customer.json"));
            List<Customer> customerList = JsonConvert.DeserializeObject<List<Customer>>(json);
            List<Invoice> invoiceList = GetAllInvoices();

            List<Customer> customers = customerList.Where(cust => cust.Name == customerName).ToList();

            List<Invoice> invoices = new List<Invoice>();
            foreach (Customer customer in customers)
            {
                invoices.AddRange(invoiceList.Where(inv => inv.CustomerID == customer.ID).AsEnumerable());
            }
            return invoices;
        }

        public List<Team> GetAllTeams()
        {
            string _jsonTeamStr = GetJSonFromFile(System.IO.Path.Combine(_jsonPath, "Team.json"));
            var format = "dd/MM/yyyy"; // your datetime format
            var dateTimeConverter = new IsoDateTimeConverter { DateTimeFormat = format };
            List<Team> teams = JsonConvert.DeserializeObject<List<Team>>(_jsonTeamStr, dateTimeConverter);

            return teams;
        }

        public List<Match>[] GetCurrentAndPreviousMatches(int range, string selectedYr)
        {
            string currFileName = "";
            string prevFileName = "";
            switch (DateTime.Now.Year - range)
            {
                case 0:
                    currFileName = "MatchCurrentYear.json";
                    prevFileName = "MatchLastYear.json";
                    break;
                case 1:
                    currFileName = "MatchLastYear.json";
                    prevFileName = "MatchSecondLastYear.json";
                    break;
                case 2:
                    currFileName = "MatchSecondLastYear.json";
                    prevFileName = "MatchThirdLastYear.json";
                    break;
                case 3:
                    currFileName = "MatchThirdLastYear.json";
                    prevFileName = "MatchFourthLastYear.json";
                    break;
                case 4:
                    currFileName = "MatchFourthLastYear.json";
                    prevFileName = "";
                    break;
                default:
                    break;
            }
            string _jsonMatchStr = GetJSonFromFile(System.IO.Path.Combine(_jsonPath, currFileName));
            var format = "dd/MM/yyyy"; // your datetime format
            var dateTimeConverter = new IsoDateTimeConverter { DateTimeFormat = format };

            List<Match> matchListCurrent = JsonConvert.DeserializeObject<List<Match>>(_jsonMatchStr, dateTimeConverter);
            string _jsonMatchStrPre = null;
            if (prevFileName != "")
            {
                _jsonMatchStrPre = GetJSonFromFile(System.IO.Path.Combine(_jsonPath, prevFileName));
            }

            List<Match> matchListPrevious = null;
            if (_jsonMatchStrPre != null)
            {
                matchListPrevious = JsonConvert.DeserializeObject<List<Match>>(_jsonMatchStrPre, dateTimeConverter);
            }
            else
            {
                matchListPrevious = new List<Match>();
            }
            UpdateDateInMatchData(matchListCurrent, matchListPrevious, selectedYr);

            List<Match>[] matches = new List<Match>[] { matchListCurrent, matchListPrevious };
            return matches;
        }

        public List<Employee> GetAllEmployees()
        {
            string jsonEmpStr = GetJSonFromFile(System.IO.Path.Combine(_jsonPath, "Employee.json"));
            var format = "dd/MM/yyyy"; // your datetime format
            var dateTimeConverter = new IsoDateTimeConverter { DateTimeFormat = format };
            List<Employee> employees = JsonConvert.DeserializeObject<List<Employee>>(jsonEmpStr, dateTimeConverter);

            return employees;
        }

        public List<AbsenceSchedule> GetAbsenceScheduleList()
        {
            string jsonAbsenceStr = GetJSonFromFile(System.IO.Path.Combine(_jsonPath, "AbsenceSchedule.json"));
            var format = "dd/MM/yyyy"; // your datetime format
            var dateTimeConverter = new IsoDateTimeConverter { DateTimeFormat = format };
            List<AbsenceSchedule> absenceSchedule = JsonConvert.DeserializeObject<List<AbsenceSchedule>>(jsonAbsenceStr, dateTimeConverter);
            return absenceSchedule;
        }

        public List<MovieRecord> GetMovieRecords()
        {
            string jsonMovieStr = GetJSonFromFile(System.IO.Path.Combine(_jsonPath, "movie_data.json"));
            List<MovieRecord> movieRecords = JsonConvert.DeserializeObject<List<MovieRecord>>(jsonMovieStr);
            return movieRecords;
        }

        public void SaveObjectToJsonFile(string filePath, object obj)
        {
            string jsonStr = JsonConvert.SerializeObject(obj);
            try
            {
              System.IO.File.WriteAllText(filePath, jsonStr);
            }
            catch
            {
            }
        }
        #endregion
    }
}
