﻿using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using Spread.Common.DataStore.Models;
using Spread.Common.DataStore.Utilities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Spread.Common.DataStore
{
    public class KPIDataSource
    {
        public KPIList GetAllKPIs()
        {
            string json = FileUtilities.LoadJSON("FinancialKPIList.json");
            KPIList kpiList = JsonConvert.DeserializeObject<KPIList>(json);
            return kpiList;
        }

        //Get all targets
        public List<TargetMaster> GetAllTargets()
        {
            string json = FileUtilities.LoadJSON("FinancialTargetMaster.json");
            var parsedObject = JObject.Parse(json);
            string businessData = parsedObject["BusinessKPI"].ToString();
            List<TargetMaster> targetMasterList = JsonConvert.DeserializeObject<List<TargetMaster>>(businessData);
            UpdateDatesInTargetData(targetMasterList);
            return targetMasterList;
        }

        private void UpdateDatesInTargetData(List<TargetMaster> targetMasterList)
        {
            int currentMonth = DateTime.Now.Month;
            int currentYr = DateTime.Now.Year;

            foreach (TargetMaster target in targetMasterList.OrderByDescending(t => t.Year).ThenByDescending(t => t.Month))
            {
                target.Month = currentMonth;
                target.Year = currentYr;
                currentMonth--;
                if (currentMonth == 0)
                {
                    currentMonth = 12;
                    currentYr--;
                }
            }
            //UpdateDataInTargetJsonFile(targetMasterList);
            //Assign 0 in data for all previous month in year
            while (currentMonth > 0 && currentMonth != 12)
            {
                TargetMaster target = new TargetMaster()
                {
                    Year = currentYr,
                    Month = currentMonth,
                    KPITargets = new List<KPIMetrics>()
                };
                foreach (var item in GetAllKPIs().Items)
                {
                    target.KPITargets.Add(new KPIMetrics()
                    {
                        KPIId = item.Key,
                        Actual = 0,
                        Target = 0
                    });
                }
                targetMasterList.Add(target);
                currentMonth--;
            }
        }
        //Update data in TargetMaster json file with date
        private void UpdateDataInTargetJsonFile(List<TargetMaster> targetMasterList)
        {
            FileUtilities.WriteToJSON("TargetMaster.json", JsonConvert.SerializeObject(targetMasterList, Formatting.Indented));
        }
    }
}
