﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.OleDb;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Spread.Common.DataStore
{
    public class OrderEntryDataSource
    {
        private AccessDataSource _dataSource;

        internal OrderEntryDataSource()
        {
            _dataSource = new AccessDataSource();
        }

        public DataTable FindCustomers(Dictionary<string, string> parameters)
        {
            string query = $"select * from Customers {BuildQuery(parameters)}";
            DataTable table = _dataSource.GetDataFrom("fpWine.mdb", query).Tables[0];
            return table;
        }

        public DataTable FindOrders(Dictionary<string, string> parameters)
        {
            string query = $"select * from Orders {BuildQuery(parameters)}";
            DataTable table = _dataSource.GetDataFrom("fpWine.mdb", query).Tables[0];
            return table;
        }

        public DataTable FindProducts(Dictionary<string, string> parameters)
        {
            string query = $"select * from Products {BuildQuery(parameters)}";
            DataTable table = _dataSource.GetDataFrom("fpWine.mdb", query).Tables[0];
            return table;
        }

        public DataTable FindEmployees(Dictionary<string, string> parameters)
        {
            string query = $"select * from Employees {BuildQuery(parameters)}";
            DataTable table = _dataSource.GetDataFrom("fpWine.mdb", query).Tables[0];
            return table;
        }

        public DataTable FindOrderDetails(Dictionary<string, string> parameters)
        {
            string query = $"select * from Orders Inner Join OrderDetails on Orders.OrderID = OrderDetails.OrderNum" +
                $" {BuildQuery(parameters)}";
            DataTable table = _dataSource.GetDataFrom("fpWine.mdb", query).Tables[0];
            return table;
        }

        public DataTable GetOpenInvoices()
        {
            string query = "Select * from Receivables where [AccountOpen] = -1 ORDER By Company Asc, OrderDate Asc";
            DataTable table = _dataSource.GetDataFrom("fpWine.mdb", query).Tables[0];
            return table;
        }

        public DataTable GetSalesReportByDate(DateTime beginDate, DateTime endDate)
        {
            string query = "select OrderDate, OrderSubtotal, Freight, OrderTax,OrderTotal from Orders where OrderDate " +
                "between #" + beginDate + "# and #" + endDate + "# Order By OrderDate ASC";
            DataTable table = _dataSource.GetDataFrom("fpWine.mdb", query).Tables[0];
            return table;
        }

        public DataTable GetSalesReportByProduct(DateTime beginDate, DateTime endDate)
        {
            string query = "select OD.ProductID, OD.Description, OD.UnitPrice, OD.Quantity, OD.LineTotal ";
            query += "from OrderDetails OD INNER JOIN Orders O on OD.OrderNum = O.OrderID ";
            query += "where O.OrderDate between #" + beginDate + "# and #" + endDate + "#" + "ORDER BY OD.ProductID";
            DataTable table = _dataSource.GetDataFrom("fpWine.mdb", query).Tables[0];
            return table;
        }

        public int AddCustomer(string companyName, string contactName, string contactTitle, string address, string address2,
            string city, string region, string postalCode, string country, string phone, string fax, string email, object discount)
        {
            string connStr = GetConnectionString(GetDBPath("fpWine.mdb"));
            string query = "insert into Customers (CompanyName, ContactName, ContactTitle, Address, Address2, City, Region, PostalCode," +
                $"Country, Phone, Fax, email, discount) Values ('{CorrectString(companyName)}', '{CorrectString(contactName)}'," +
                $"'{CorrectString(contactTitle)}', '{CorrectString(address)}', '{CorrectString(address2)}'," +
                $"'{CorrectString(city)}', '{CorrectString(region)}', '{CorrectString(postalCode)}', '{CorrectString(country)}'," +
                $"'{CorrectString(phone)}', '{CorrectString(fax)}', '{CorrectString(email)}', '{CorrectValue(discount)}')";

            int result = -1;
            using (OleDbConnection connection = new OleDbConnection(connStr))
            {
                using (OleDbCommand command = new OleDbCommand(query, connection))
                {
                    connection.Open();
                    result = command.ExecuteNonQuery();
                    connection.Close();
                }
            }
            return result;
        }

        public int UpdateCustomer(int custID, string companyName, string contactName, string contactTitle, string address, string address2,
            string city, string region, string postalCode, string country, string phone, string fax, string email, object discount)
        {
            string connStr = GetConnectionString(GetDBPath("fpWine.mdb"));
            string query = $"update Customers set CompanyName = '{CorrectString(companyName)}', ContactName = '{CorrectString(contactName)}'," +
                $"ContactTitle = '{CorrectString(contactTitle)}', Address = '{CorrectString(address)}'," +
                $"Address2 = '{CorrectString(address2)}', City = '{CorrectString(city)}', Region = '{CorrectString(region)}'," +
                $"PostalCode = '{CorrectString(postalCode)}', Country = '{CorrectString(country)}', Phone = '{CorrectString(phone)}'," +
                $"Fax = '{CorrectString(fax)}', email = '{CorrectString(email)}', discount = '{CorrectValue(discount)}' " +
                $"where ID = {custID}";

            int result = -1;
            using (OleDbConnection connection = new OleDbConnection(connStr))
            {
                using (OleDbCommand command = new OleDbCommand(query, connection))
                {
                    connection.Open();
                    result = command.ExecuteNonQuery();
                    connection.Close();
                }
            }
            return result;
        }

        public int AddOrder(int CustomerID, string SalesPerson, string ShipMethod, decimal Freight, string ShipCompany, string ShipName, string ShipAddress, string ShipAddress2, string ShipCity, string ShipRegion,
            string ShipPostalCode, string ShipCountry, string ShipPhone, string BillCompany, string BillName, string BillAddress, string BillAddress2, string BillCity, string BillRegion, string BillPostalCode,
            string BillCountry, string BillPhone, string PaymentMethod, string PaymentNumber, string CardExpDate, string CardholderName, decimal OrderSubTotal, decimal OrderTotal, decimal AmtPaid, decimal BalanceDue,
            decimal OrderTax)
        {
            string connStr = GetDBPath("fpWine.mdb");
            string query = $"insert into Orders (CustomerID, SalesPerson, OrderDate, ShipMethod, Freight, ShipCompany," +
                $"ShipName, ShipAddress, ShipAddress2, ShipCity, ShipRegion, ShipPostalCode, ShipCountry, ShipPhone," +
                $"BillCompany, BillName, BillAddress, BillAddress2, BillCity, BillRegion, BillPostalCode, BillCountry," +
                $"BillPhone, PaymentMethod, PaymentNumber, CardExpDate, CardholderName, OrderSubTotal, OrderTotal, AmtPaid," +
                $"BalanceDue, OrderTax) Values ('{CustomerID.ToString()}', '{SalesPerson}', '{DateTime.Today.ToString()}'," +
                $"'{ShipMethod}', '{CorrectValue(Freight)}', '{CorrectString(ShipCompany)}', '{CorrectString(ShipName)}'," +
                $"'{CorrectString(ShipAddress)}', '{CorrectString(ShipAddress2)}', '{CorrectString(ShipCity)}'," +
                $"'{CorrectString(ShipRegion)}', '{CorrectString(ShipPostalCode)}', '{CorrectString(ShipCountry)}'," +
                $"'{CorrectString(ShipPhone)}', '{CorrectString(BillCompany)}', '{CorrectString(BillName)}', '{CorrectString(BillAddress)}'," +
                $"'{CorrectString(BillAddress2)}', '{CorrectString(BillCity)}', '{CorrectString(BillRegion)}'," +
                $"'{CorrectString(BillPostalCode)}', '{CorrectString(BillCountry)}', '{CorrectString(BillPhone)}'," +
                $"'{PaymentMethod}', '{CorrectString(PaymentNumber)}', '{CorrectString(CardExpDate)}', '{CorrectString(CardholderName)}'," +
                $"'{CorrectValue(OrderSubTotal)}', '{CorrectValue(OrderTotal)}', '{CorrectValue(AmtPaid)}'," +
                $"'{CorrectValue(BalanceDue)}', '{CorrectValue(OrderTax)}')";

            int result = -1;
            using (OleDbConnection connection = new OleDbConnection(connStr))
            {
                using (OleDbCommand command = new OleDbCommand(query, connection))
                {
                    result = command.ExecuteNonQuery();
                }
            }
            return result;
        }

        public int UpdateOrder(int OrderID, string SalesPerson, string ShipMethod, decimal Freight, string ShipCompany, string ShipName, string ShipAddress, string ShipAddress2, string ShipCity, string ShipRegion,
            string ShipPostalCode, string ShipCountry, string ShipPhone, string BillCompany, string BillName, string BillAddress, string BillAddress2, string BillCity, string BillRegion, string BillPostalCode,
            string BillCountry, string BillPhone, string PaymentMethod, string PaymentNumber, string CardExpDate, string CardholderName, decimal OrderSubTotal, decimal OrderTotal, decimal AmtPaid, decimal BalanceDue,
            decimal OrderTax)
        {
            string connStr = GetConnectionString(GetDBPath("fpWine.mdb"));
            string query = $"update Orders set SalesPerson = '{SalesPerson}', ShipMethod = '{ShipMethod}', Freight = '{CorrectValue(Freight)}'," +
                $"ShipCompany = '{CorrectString(ShipCompany)}', ShipName = '{CorrectString(ShipName)}', ShipAddress = '{CorrectString(ShipAddress)}'," +
                $"ShipAddress2 = '{CorrectString(ShipAddress2)}', ShipCity = '{CorrectString(ShipCity)}', ShipRegion = '{CorrectString(ShipRegion)}'," +
                $"ShipPostalCode = '{CorrectString(ShipPostalCode)}', ShipCountry = '{CorrectString(ShipCountry)}'," +
                $"ShipPhone = '{CorrectString(ShipPhone)}', BillCompany = '{CorrectString(BillCompany)}', BillName = '{CorrectString(BillName)}'," +
                $"BillAddress = '{CorrectString(BillAddress)}', BillAddress2 = '{CorrectString(BillAddress2)}', BillCity = '{CorrectString(BillCity)}'," +
                $"BillRegion = '{CorrectString(BillRegion)}', BillPostalCode = '{CorrectString(BillPostalCode)}', BillCountry = '{CorrectString(BillCountry)}'," +
                $"BillPhone = '{CorrectString(BillPhone)}', PaymentMethod = '{PaymentMethod}', PaymentNumber = '{CorrectString(PaymentNumber)}'," +
                $"CardExpDate = '{CorrectString(CardExpDate)}', CardholderName = '{CorrectString(CardholderName)}', OrderSubTotal = '{CorrectValue(OrderSubTotal)}'," +
                $"OrderTotal = '{CorrectValue(OrderTotal)}', AmtPaid = '{CorrectValue(AmtPaid)}', BalanceDue = '{CorrectValue(BalanceDue)}'," +
                $"OrderTax = '{CorrectValue(OrderTax)}' where OrderID = {OrderID}";

            int result = -1;
            using (OleDbConnection connection = new OleDbConnection(connStr))
            {
                using (OleDbCommand command = new OleDbCommand(query, connection))
                {
                    connection.Open();
                    result = command.ExecuteNonQuery();
                    connection.Close();
                }
            }
            return result;
        }

        public int DeleteLineItem(int orderNum)
        {
            string connStr = GetConnectionString(GetDBPath("fpWine.mdb"));
            string query = $"delete from OrderDetails where OrderNum = {orderNum}";
            int result = -1;
            using (OleDbConnection connection = new OleDbConnection(connStr))
            {
                using (OleDbCommand command = new OleDbCommand(query, connection))
                {
                    connection.Open();
                    result = command.ExecuteNonQuery();
                    connection.Close();
                }
            }
            return result;
        }

        public int AddLineItem(int OrderNum, int ProductID, string Description, decimal UnitPrice, decimal LineTotal, int Quantity, decimal Discount)
        {
            string connStr = GetConnectionString(GetDBPath("fpWine.mdb"));
            string query = $"insert into OrderDetails (OrderNum,ProductID,Description,UnitPrice,LineTotal,Quantity,Discount) " +
                $"Values ('{OrderNum}', '{CorrectValue(ProductID)}', '{CorrectString(Description)}', '{CorrectValue(UnitPrice)}'," +
                $"'{CorrectValue(LineTotal)}', '{CorrectValue(Quantity)}', '{CorrectValue(Discount)}')";
            int result = -1;
            using (OleDbConnection connection = new OleDbConnection(connStr))
            {
                using (OleDbCommand command = new OleDbCommand(query, connection))
                {
                    connection.Open();
                    result = command.ExecuteNonQuery();
                    connection.Close();
                }
            }
            return result;
        }

        private string BuildQuery(Dictionary<string, string> parameters)
        {
            string query = string.Empty;
            if (parameters.Any())
                query = "where ";

            List<string> args = new List<string>();

            foreach (var entry in parameters)
            {
                if (entry.Key.Contains("ID") || entry.Key.Contains("Num"))
                    args.Add($"{entry.Key} = {entry.Value}");
                else
                    args.Add($"{entry.Key} like '{entry.Value}%'");
            }

            query += String.Join(" and ", args);
            return query;
        }

        private string GetConnectionString(string dbPath)
        {
            return String.Format("Provider=Microsoft.Jet.OLEDB.4.0; Data Source={0}", dbPath);
        }

        private string GetDBPath(string dbName)
        {
            string dbPath = string.Empty;
            string fileName = string.Format("{0}", dbName);
            Microsoft.Win32.RegistryKey regKey = Microsoft.Win32.Registry.ClassesRoot.OpenSubKey("FarPoint.Win.Spread\\Common7");
            if (regKey != null)
                dbPath = (string)regKey.GetValue("v" + GCVersionUtil.ProductVersion);
            if (!System.IO.File.Exists(dbPath + fileName))
                dbPath = Environment.GetFolderPath(Environment.SpecialFolder.ProgramFiles) + GCVersionUtil.SPREAD_INSTALLATION_FOLDER + "\\Common\\";
            if (!System.IO.File.Exists(dbPath + fileName))
                dbPath = "Data\\";
            dbPath += fileName;
            return dbPath;
        }

        public string CorrectString(object str)
        {
            string newstr;
            if (str == null || str == DBNull.Value)
                str = string.Empty;

            newstr = ((string)str).Replace("'", "''");
            newstr = (newstr).Replace(" ", "");
            return newstr;
        }

        public double CorrectValue(object str)
        {
            double newstr = 0;
            if (str == null || str == DBNull.Value)
                newstr = 0;
            else
                newstr = Convert.ToDouble(str);

            return newstr;
        }
    }
}
