﻿using System;
using System.Collections.Generic;
using Spread.Common.DataStore.Models;

namespace Spread.Common.DataStore.DataSourceEntities
{
    public class StudentCalendarDataSource
    {
        private List<ScheduleEntry> _schedule;
        private List<Assignment> _assignments;

        public static List<string> Subjects { get; }
        public static List<string> AssignmentTypes { get; }

        private List<ScheduleEntry> Schedule
        {
            get
            {
                if (_schedule == null)
                    _schedule = new List<ScheduleEntry>();
                return _schedule;
            }
            set
            {
                _schedule = value;
            }
        }

        private List<Assignment> Assignments
        {
            get
            {
                if (_assignments == null)
                    _assignments = new List<Assignment>();
                return _assignments;
            }
            set
            {
                _assignments = value;
            }
        }

        static StudentCalendarDataSource()
        {
            Subjects = new List<string> { "English", "History", "Physics", "Chemistry", "Maths",
                "Programming", "Psychology", "French", "Art" };
            AssignmentTypes = new List<string> { "Test", "Homework" };
        }

        private List<Assignment> UpdatedAssignments(int year)
        {
            Assignments.ForEach(assignment => assignment.Date = new DateTime(year, assignment.Date.Month,
                assignment.Date.Day));
            return Assignments;
        }

        public List<ScheduleEntry> GetSchedule()
        {
            if (Schedule.Count != 50)
                _schedule = new List<ScheduleEntry>();
            Random random = new Random();

            for (int i = 0; i < 50; ++i)
            {
                ScheduleEntry entry = new ScheduleEntry(Subjects[random.Next(0, Subjects.Count)],
                    (DayOfWeek)(i % 5 + 1), 8 + i / 5);
                Schedule.Add(entry);
            }

            return Schedule;
        }

        public List<Assignment> GetAssignments(int year)
        {
            if (Assignments.Count != 0)
                return UpdatedAssignments(year);

            Random random = new Random();

            DateTime date = new DateTime(year, 1, 1).AddDays(random.Next(0, 8));
            DateTime endDate = new DateTime(year, 12, 31);
            while (date <= endDate)
            {
                Assignment assignment = new Assignment(date, String.Join(": ", Subjects[random.Next(0, Subjects.Count)],
                    AssignmentTypes[random.Next(0, AssignmentTypes.Count)]));
                Assignments.Add(assignment);

                date = date.AddDays(random.Next(4, 12));
            }

            return Assignments;
        }
    }
}
