﻿using Newtonsoft.Json;
using Spread.Common.DataStore.Models;
using Spread.Common.DataStore.Utilities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Spread.Common.DataStore.DataSourceEntities
{
    public class StudentDashboardDataSource
    {
        #region Public methods
        // Get a List of all Students
        public List<Student> GetAllStudents()
        {
            string json = FileUtilities.LoadJSON("Students.json");
            List<Student> students = JsonConvert.DeserializeObject<List<Student>>(json);
            return students;
        }

        // Get a List of all Absence Records
        public List<AbsentRecords> GetAllAbsentRecords()
        {
            string json = FileUtilities.LoadJSON("AbsentRecords.json");
            List<AbsentRecords> absentRecords = JsonConvert.DeserializeObject<List<AbsentRecords>>(json);

            absentRecords = absentRecords.OrderBy(r => r.SessionID).ToList();
            int diff = DateTime.Now.Year - 2018;

            // Update dates of each record to keep it current
            if (diff != 0)
            {
                foreach (var list in absentRecords)
                {
                    list.StudentRecords = list.StudentRecords.OrderBy(r => r.StudentID).ThenBy(r => r.Date).ToList();
                    foreach (var record in list.StudentRecords)
                    {
                        if (DateTime.IsLeapYear(record.Date.Year) && record.Date.Month == 2 && record.Date.Day == 29)
                            record.Date = new DateTime(record.Date.Year + diff, record.Date.Month + 1, 1);
                        else
                            record.Date = new DateTime(record.Date.Year + diff, record.Date.Month, record.Date.Day);
                    }
                }
            }

            return absentRecords;
        }

        // Get a List of all Tardy Records
        public List<TardyRecords> GetAllTardyRecords()
        {
            string json = FileUtilities.LoadJSON("TardyRecords.json");
            List<TardyRecords> tardyRecords = JsonConvert.DeserializeObject<List<TardyRecords>>(json);

            tardyRecords = tardyRecords.OrderBy(r => r.SessionID).ToList();
            int diff = DateTime.Now.Year - 2018;

            // Update dates of each record to keep it current
            if (diff != 0)
            {
                foreach (var list in tardyRecords)
                {
                    list.StudentRecords = list.StudentRecords.OrderBy(r => r.StudentID).ThenBy(r => r.Date).ToList();
                    foreach (var record in list.StudentRecords)
                    {
                        if (DateTime.IsLeapYear(record.Date.Year) && record.Date.Month == 2 && record.Date.Day == 29)
                            record.Date = new DateTime(record.Date.Year + diff, record.Date.Month + 1, 1);
                        else
                            record.Date = new DateTime(record.Date.Year + diff, record.Date.Month, record.Date.Day);
                    }
                }
            }

            return tardyRecords;
        }

        // Get a List of all Score Records
        public List<ScoreRecords> GetAllScoreRecords()
        {
            string json = FileUtilities.LoadJSON("ScoreRecords.json");
            List<ScoreRecords> scoreRecords = JsonConvert.DeserializeObject<List<ScoreRecords>>(json);
            return scoreRecords;
        }

        // Get a List of all Term Sessions
        public List<TermSession> GetAllTermSessions()
        {
            string json = FileUtilities.LoadJSON("TermSessions.json");
            List<TermSession> termSessions = JsonConvert.DeserializeObject<List<TermSession>>(json);

            // Update Year of each Term Session to keep it current
            int diff = DateTime.Now.Year - 2018;
            if (diff != 0)
            {
                foreach (var session in termSessions)
                {
                    session.Year += diff;
                }
            }

            return termSessions;
        }
        #endregion
    }
}
