﻿using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Spread.Common.DataStore.Models
{
    public class FinancialKPI
    {
        public string Name { get; set; }
        public KPIType Type { get; set; }
    }

    public enum KPIType : byte
    {
        Asset = 0,
        Liability = 1
    }

    public class FinancialKPIList
    {
        #region Properties
        [JsonProperty("FinancialKPIItems")]
        public Dictionary<int, FinancialKPI> Items { get; set; }
        #endregion

        #region ctor
        public FinancialKPIList()
        {
            Items = new Dictionary<int, FinancialKPI>();
        }
        #endregion
    }

    public class FinancialKPIMetrics
    {
        #region Properties
        public int KPIId { get; set; }
        public double Actual { get; set; }
        public double Target { get; set; }
        #endregion
    }

    public class FinancialTargetMaster
    {
        #region Fields
        private int _month;
        private int _year;
        #endregion

        #region Properties
        [JsonProperty("FinancialKPITargets")]
        public List<FinancialKPIMetrics> KPITargets { get; set; }

        public int Month
        {
            get { return _month; }
            set
            {
                if (value < 1 || value > 12)
                    throw new NotSupportedException("Value not supported.\nExpected Value: 1 to 12\nActual Value: " + value);
                _month = value;
            }
        }

        public int Year
        {
            get { return _year; }
            set
            {
                if (value < 1000 || value > 9999)
                    throw new NotSupportedException("Value not supported.\nExpected Value: 1000 to 9999\nActual Value: " + value);
                _year = value;
            }
        }
        #endregion

        #region ctor
        public FinancialTargetMaster()
        {
            KPITargets = new List<FinancialKPIMetrics>();
        }
        #endregion
    }
}
