﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace Spread.ControlExplorer.Components
{
    public partial class Header : UserControl
    {
        public delegate void HeaderNavigationHandler(object sender, HeaderNavigationEventArgs e);
        public event HeaderNavigationHandler OnHeaderNavigate;

        public string Title
        {
            get
            {
                return _lblName.Text;
            }
            set
            {
                _lblName.Text = value;
            }
        }

        public Image Image
        {
            get
            {
                return _pbLogo.Image;
            }
            set
            {
                _pbLogo.Image = value;
            }
        }
        
        public Color HighlightColor
        {
            get
            {
                return panel1.BackColor;
            }
            set
            {
                panel1.BackColor = value;
            }
        }

        public Color ButtonBackColor
        {
            get
            {
                return btnDownload.BackColor;
            }
            set
            {
                btnDownload.BackColor = value;
            }
        }

        public Color ButtonForeColor
        {
            get
            {
                return btnDownload.ForeColor;
            }
            set
            {
                btnDownload.ForeColor = value;
            }
        }
        
        public string BreadCrumbPath
        {
            get
            {
                return _breadCrumb.Text;
            }
            set
            {
                if(_breadCrumb.Text != value)
                {
                    _breadCrumb.Text = value;
                    foreach(var path in _breadCrumb.Text.Split('\\'))
                    {
                        _breadCrumb.AddPath(path, null);
                    }
                }
            }
        }

        public string DownloadTitle
        {
            get
            {
                return btnDownload.Text;
            }
            set
            {
                btnDownload.Text = value;
            }
        }

        public string DownloadLink { get; set; }

        public Header()
        {
            InitializeComponent();
        }

        protected override void OnBackColorChanged(EventArgs e)
        {
            base.OnBackColorChanged(e);
            foreach(var control in this.Controls)
            {
                (control as Control).BackColor = this.BackColor;
            }
        }

        protected override void OnForeColorChanged(EventArgs e)
        {
            base.OnForeColorChanged(e);
            _breadCrumb.ForeColor = _lblName.ForeColor = this.ForeColor;
        }

        private void btnPrevious_Click(object sender, EventArgs e)
        {
            OnHeaderNavigate?.Invoke(this, new HeaderNavigationEventArgs(NavigationDirectionEnum.Previous));
        }

        private void btnNext_Click(object sender, EventArgs e)
        {
            OnHeaderNavigate?.Invoke(this, new HeaderNavigationEventArgs(NavigationDirectionEnum.Next));
        }

        private void btnDownload_Click(object sender, EventArgs e)
        {
            System.Diagnostics.Process.Start(DownloadLink);
        }

        private void NavMouseEnter(object sender, EventArgs e)
        {
            this.Cursor = Cursors.Hand;
        }

        private void NavMouseLeave(object sender, EventArgs e)
        {
            this.Cursor = Cursors.Default;
        }
    }

    public class HeaderNavigationEventArgs : EventArgs
    {
        public NavigationDirectionEnum Direction { get; set; }

        public HeaderNavigationEventArgs(NavigationDirectionEnum dir)
        {
            this.Direction = dir;
        }

    }

    public enum NavigationDirectionEnum
    {
        Next,
        Previous,
        First,
        Last
    }
}
