﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Spread.Common.Features.Sample;
using Spread.Common.Features.Utilities;

namespace Spread.ControlExplorer.Components
{
    public partial class SampleHost : UserControl
    {
        public delegate void SampleAddHandler(object sender, EventArgs e);
        public event SampleAddHandler OnSampleAdded;
        private bool formatCode = false;  
    public SampleHost()
        {
            InitializeComponent();
            ToggleAdditionalInformation(false);           
             
        }


        private void TabControl_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (!formatCode && tabControl.TabPages[tabControl.SelectedIndex] == codeTab)
            {
                formatCode = true;
                rtbCode.FormatRfToCS();
            }
        }



        public Control Sample
        {
            get
            {
                return this.pnlHost.Controls["sample"];
            }
            set
            {
                if(value != null)
                {
                    tabControl.SelectedIndexChanged -= TabControl_SelectedIndexChanged;
                    var old = this.pnlHost.Controls["sample"];
                    if(old!=null)
                        old.Name = "oldSample";
                    value.Name = "sample";
                    value.Dock = DockStyle.Fill;
                    this.pnlHost.Controls.Add(value);
                    this.pnlHost.Controls.RemoveByKey("oldSample");
                    this.tabControl.SelectedIndex = 0;
                    if (this.OnSampleAdded != null)
                    {
                        this.OnSampleAdded(this, EventArgs.Empty);
                    }
                    tabControl.SelectedIndexChanged += TabControl_SelectedIndexChanged;
        }
            }
        }
        
        public bool ShowBorder
        {
            get
            {
                return this.pnlHost.ShowBorder;
            }
            set
            {
                this.pnlHost.ShowBorder = value;
            }
        }

        string _Description = null;
        public string Description
        {
            get
            {
                return rtbDesc.Rtf;
            }
            set
            {
                _Description = value;
                if (!string.IsNullOrEmpty(value))
                {
                    rtbDesc.Clear();
                    try
                    {
                        rtbDesc.Rtf = value;
                    }
                    catch(System.ArgumentException)
                    {
                        rtbDesc.Text = value;
                    }
                    if (!this.tabControl.TabPages.Contains(descriptionTab))
                    {
                        this.tabControl.TabPages.Insert(0,descriptionTab);
                    }
                    ToggleAdditionalInformation();
                }
                else
                {
                    if (this.tabControl.TabPages.Contains(descriptionTab))
                    {
                        this.tabControl.TabPages.Remove(descriptionTab);
                    }
                    ToggleAdditionalInformation();
                }
            }
        }

        string _Code = null;
        public string Code
        {
            get
            {
                return rtbCode.Text;
            }
            set
            {
                _Code = value;
                 formatCode = false;
                if (!string.IsNullOrEmpty(value))
                {
                    rtbCode.Clear();
                    rtbCode.Text = value;
                    //rtbCode.FormatRfToCS();
                    if (!this.tabControl.TabPages.Contains(codeTab))
                    {
                        this.tabControl.TabPages.Add(codeTab);
                    }
                    ToggleAdditionalInformation();
                }
                else
                {
                    if (this.tabControl.TabPages.Contains(codeTab))
                    {
                        this.tabControl.TabPages.Remove(codeTab);
                    }
                    ToggleAdditionalInformation();
                }
            }
        }

        public void ToggleAdditionalInformation(bool show = true)
        {
            tLP.RowStyles[0].SizeType = SizeType.Percent;
            tLP.RowStyles[1].SizeType = SizeType.Percent;
            int _m = 60;
            int _a = 40;
            if (show)
            {
                show = !string.IsNullOrWhiteSpace(_Description) || !string.IsNullOrWhiteSpace(_Code);
            }
            if (!show)
            {
                _m = 100;
                _a = 0;
            }
            tLP.RowStyles[0].Height = _m;
            tLP.RowStyles[1].Height = _a;
            
        }
    }
}
