﻿
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Spread.Common.Features.Sample;
using Spread.Common.Features.Explorer.CustomControls;
using Spread.ControlExplorer.Components;
using Spread.Common.Features.Utilities;

namespace Spread.ControlExplorer.Components
{
    public partial class Sidebar : UserControl
    {
        public delegate void SidebarNavigationHandler(object sender, SidebarNavigationEventArgs e);
        public event SidebarNavigationHandler OnSidebarNavigate;

        List<BaseSample> dataSource = new List<BaseSample>();
        public List<BaseSample> DataSource
        {
            get
            {
                return dataSource;
            }
            set
            {
                dataSource = value;
                UpdateTreeview();
            }
        }

        private void UpdateTreeview()
        {
            StackTreeControl.ImageList = new Dictionary<string, Image>();
            StackTreeControl.ImageList.Add("Quickstart", Spread.Common.Features.Properties.Resources.fighter_jet_silhouette);
            StackTreeControl.ImageList.Add("Home", Spread.Common.Features.Properties.Resources.home);
            StackTreeControl.ImageList.Add("Plus", Spread.Common.Features.Properties.Resources.plus_symbol_in_a_rounded_black_square);
            StackTreeControl.ImageList.Add("Minus", Spread.Common.Features.Properties.Resources.minus_sign_inside_a_black_rounded_square_shape);
            _tv.Nodes.Clear();
            var home = _tv.Nodes.Add("Home", "Home", "Home");
            var quickStart = _tv.Nodes.Add("Quickstart", "Quickstart", "Quickstart");
            var examplesNode = _tv.Nodes.Add("Examples", "Showcase", "Plus", "Minus");
            var featuresNode = _tv.Nodes.Add("Features", "Features", "Plus", "Minus");

            home.Tag = dataSource.FirstOrDefault(d => d.Title == "Home");
            quickStart.Tag = dataSource.FirstOrDefault(d => d.Title == "Quickstart");
            examplesNode.Tag = new ExampleSample { Title = "Showcase", IsGroup = true };

            foreach (var ds in dataSource.Where(d => d.NotASample == false))
            {
                if (typeof(ExampleSample) == ds.GetType())
                {
                    var node = examplesNode.Nodes.Add(
                        (ds as ExampleSample).UniqueName,
                        (ds as ExampleSample).Title
                        );
                    node.Tag = ds;
                    node.IsSearchable = true;
                    node.SearchString = (ds as ExampleSample).Title + "\n" + (ds as ExampleSample).Description.FromRTFToStrnig();
                    node.Status = ds.Status;
                }
                else if (typeof(FeatureSample) == ds.GetType())
                {
                    var featureTree = (ds as FeatureSample).FeatureName.Split(',');
                    var parent = featuresNode;
                    foreach (var feature in featureTree)
                    {
                        var fNode = parent.FindNode(feature);
                        if (fNode == null)
                        {
                            fNode = parent.Nodes.Add(feature, feature);
                            fNode.CollapsedImage = Spread.Common.Features.Properties.Resources.arrowhead_pointing_to_the_right;
                            fNode.ExpandedImage = Spread.Common.Features.Properties.Resources.sort_down;
                            fNode.Tag = new BaseSample
                            {
                                Title = (ds as FeatureSample).FeatureName,
                                IsGroup = true
                            };
                        }
                        parent = fNode;
                    }
                    var featureNode = featuresNode.FindNode(featureTree.Last(), true);
                    var node = featureNode.Nodes.Add(
                        (ds as FeatureSample).UniqueName,
                        (ds as FeatureSample).Title);
                    node.Tag = ds;
                    node.IsSearchable = true;
                    node.SearchString = (ds as FeatureSample).Title + "\n" + (ds as FeatureSample).Description.FromRTFToStrnig() + "\n" + (ds as FeatureSample).FeatureName;
                    node.Status = ds.Status;
                    if (string.IsNullOrEmpty(parent.Status) && !string.IsNullOrEmpty(ds.Status))
                      parent.Status = node.Status;
        }
            }

            _tv.SelectedNode = _tv.Nodes[0];
        }

        public Sidebar()
        {
            InitializeComponent();
        }

        public void SetSelectedSample(BaseSample sample)
        {
            if (sample != null)
                _tv.SelectedNode = _tv.FindNode(sample.UniqueName, true);
        }

        private void Sidebar_Load(object sender, EventArgs e)
        {
            this.textBoxEx1.Refresh();
        }

        private void textBoxEx1_TextChanged(object sender, EventArgs e)
        {
            this._tv.FilterString = this.textBoxEx1.Text;
        }

        private void _tv_StackNodeClick(object sender, StackNodeEventArgs e)
        {
            if (OnSidebarNavigate != null)
            {
                OnSidebarNavigate(this, new SidebarNavigationEventArgs(e.Node.Tag as BaseSample));
            }
        }
    }

    public class SidebarNavigationEventArgs : EventArgs
    {
        public BaseSample Sample { get; set; }

        public SidebarNavigationEventArgs(BaseSample sample)
        {
            this.Sample = sample;
        }
    }
}
