﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Spread.Common.Features.Utilities;

namespace Spread.ControlExplorer.Components
{
    public partial class StackTreeControl : UserControl
    {
        public event StackNodeEventHandler SelectionChanged;
        public static Dictionary<string, Image> ImageList { get; set; } = new Dictionary<string, Image>();

        private string _filterString = "";
        private StackNodeControl _selectedNode;
        private StackedNodeCollection _nodes;

        public string FilterString
        {
            get
            {
                return _filterString;
            }
            set
            {
                if (string.IsNullOrEmpty(value) || string.IsNullOrWhiteSpace(value))
                {
                    _filterString = "";
                }
                else
                {
                    _filterString = value;
                }
                this.SuspendDrawing();
                foreach (StackNodeControl node in this.Nodes)
                {
                    node.Expand(GetFilteredContentOnlyChild(node));
                }
                this.ResumeDrawing();
            }
        }

        private bool GetFilteredContentOnlyChild(StackNodeControl node)
        {
            if (node.IsSearchable)
            {
                if (node.SearchString != null)
                {
                    node.ShouldBeSeen = node.SearchString.ToLower().Contains(_filterString.ToLower());
                    return node.ShouldBeSeen;
                }
                return true;
            }
            else
            {
                if(node.Nodes.Count > 0)
                {
                    var somethingIsVisble = false;
                    foreach (StackNodeControl n in node.Nodes)
                    {
                        somethingIsVisble |= GetFilteredContentOnlyChild(n);
                    }
                    node.ShouldBeSeen = somethingIsVisble;
                    return node.ShouldBeSeen;
                }
                else
                {
                    return true;
                }
                
            }
        }

        public Color HoverBackColor { get; set; }
        public Color HoverForeColor { get; set; }

        public StackNodeControl SelectedNode
        {
            get
            {
                return _selectedNode;
            }
            set
            {
                if (value != null && _selectedNode != value)
                {
                    if (_selectedNode != null)
                    {
                        _selectedNode.SelectionColor = Color.White;
                    }
                    _selectedNode = value;
                    _selectedNode.SelectionColor = Color.FromArgb(241, 241, 241);
                    if (SelectionChanged != null)
                    {
                        SelectionChanged(this, new StackNodeEventArgs(_selectedNode));
                    }
                }
            }
        }

        public StackedNodeCollection Nodes
        {
            get
            {
                if (_nodes == null)
                {
                    _nodes = new StackedNodeCollection();
                    _nodes.NodeAdded -= OnNodeAdded;
                    _nodes.NodeAdded += OnNodeAdded;
                }
                return _nodes;
            }
        }

        private void OnNodeAdded(object sender, StackNodeEventArgs e)
        {
            var node = e.Node;
            node.HoverBackColor = this.HoverBackColor;
            node.HoverForeColor = this.HoverForeColor;
            node.ForeColor = this.ForeColor;
            node.BackColor = this.BackColor;
            node.Level = 0;
            node.Width = this.Width;
            node.StackedTree = this;
            node.ParentNode = null;
            node.IsVisible = true;
            this.pnlMain.Controls.Add(node);
            node.StackNodeClick += (s, ev) => this.OnStackNodeClick(ev);
        }

        public StackTreeControl()
        {
            InitializeComponent();
        }

        public StackNodeControl FindNode(string key, bool searchAllChild = true)
        {
            StackNodeControl n = null;
            foreach (StackNodeControl node in this.Nodes)
            {
                if (node.Key == key)
                    n = node;
                else
                    n = node.FindNode(key);
                if (n == null)
                    continue;
                else
                    break;
            }
            return n;
        }

        public void FilterNode(string filterText)
        {
            foreach (StackNodeControl node in this.Nodes)
            {
                if (node.IsSearchable)
                {

                }
            }
        }

        public void OnStackNodeClick(StackNodeEventArgs e)
        {
            this.SelectedNode = e.Node;
        }
    }
}
