﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Collections.ObjectModel;
using System.Collections.Specialized;

namespace Spread.ControlExplorer.Components
{
    public partial class TileGroup : UserControl
    {
        public string GroupName { get; set; }

        public ObservableCollection<TileView> Tiles { get; private set; }
        public TileGroup()
        {
            InitializeComponent();
            Tiles = new ObservableCollection<TileView>();
            Tiles.CollectionChanged += OnTilesCollectionChanged;
        }

        public TileGroup(string groupName) : this()
        {
            GroupName = groupName;
        }

        private void OnTilesCollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {

            if (e.NewItems != null && e.NewItems.Count > 0)
            {
                foreach (TileView newItem in e.NewItems)
                    flowLayoutPanel1.Controls.Add(newItem);
            }
            if (e.OldItems != null && e.OldItems.Count > 0)
            {
                foreach (TileView oldItem in e.OldItems)
                    flowLayoutPanel1.Controls.Remove(oldItem);
            }
        }

        private Color _HoverColor;
        public Color HoverColor
        {
            get
            {
                if (this.Parent.Parent.Parent is TileControl)
                {
                    return (this.Parent.Parent.Parent as TileControl).HoverColor;
                }
                else
                {
                    return _HoverColor;
                }
            }
            set
            {
                if (_HoverColor != value)
                {
                    _HoverColor = value;
                }
            }
        }

        public override Size GetPreferredSize(Size proposedSize)
        {
            var size = proposedSize;
            if (Parent != null && Tiles.Count > 0)
            {
                var tile = Tiles.First();
                var tileHeight = tile.Height + tile.Margin.Vertical + tile.Padding.Vertical;
                var tileWidth = tile.Width + tile.Margin.Horizontal + tile.Padding.Horizontal;
                var scrollWidth = (Parent as Panel).VerticalScroll.Visible ? SystemInformation.VerticalScrollBarWidth : 0;
                size.Width = Parent.Width - Parent.Margin.Horizontal - Parent.Padding.Horizontal - scrollWidth;
                var colsCount = size.Width / tileWidth;
                var rowCount = Math.Ceiling((double)Tiles.Count / colsCount);
                size.Height = (int)(tileHeight * rowCount);
            }
            return size;
        }
    }
}
