﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Spread.Common.Features.Core;

namespace Spread.ControlExplorer.Components
{
    public partial class TileView : UserControl
    {
        private Color _HoverColor;
        public Color HoverColor
        {
            get
            {
                if(this.Parent.Parent is TileGroup)
                {
                    return (this.Parent.Parent as TileGroup).HoverColor;
                }
                else
                {
                    return _HoverColor;
                }
            }
            set
            {
                if (_HoverColor != value)
                {
                    _HoverColor = value;
                }
            }
        }
        public new string Text
        {
            get { return lblCardName.Text; }
            set { lblCardName.Text = value; }
        }

        private Image img;
        public Image Image
        {
            get { return img; }
            set
            {
                img = value;
                picBox.Image = GetThumbnail(img);
            }
        }

        public TileView()
        {
            InitializeComponent();
            AutoSize = false;
            Margin = new Padding(10);
            lblCardName.Click += (s, e) => this.OnClick(e);
            picBox.Click += (s, e) => this.OnClick(e);
            pnlForPb.Click += (s, e) => this.OnClick(e);
            lblCardName.MouseEnter += (s, e) => this.OnMouseEnter(e);
            picBox.MouseEnter += (s, e) => this.OnMouseEnter(e);
            pnlForPb.MouseEnter += (s, e) => this.OnMouseEnter(e);
            lblCardName.MouseLeave += (s, e) => this.OnMouseLeave(e);
            picBox.MouseLeave += (s, e) => this.OnMouseLeave(e);
            pnlForPb.MouseLeave += (s, e) => this.OnMouseLeave(e);
        }
        
        private Image GetThumbnail(Image image)
        {
            return image.GetThumbnailImage(picBox.PreferredSize.Width, picBox.PreferredSize.Height, null, new IntPtr());
        }

        protected override void OnMouseEnter(EventArgs e)
        {
            base.OnMouseEnter(e);
            this.Cursor = Cursors.Hand;
            this.lblCardName.ForeColor = HoverColor;
        }

        protected override void OnMouseLeave(EventArgs e)
        {
            base.OnMouseLeave(e);
            this.Cursor = Cursors.Default;
            this.lblCardName.ForeColor = this.Parent.ForeColor;
        }

        private void pnlForPb_Paint(object sender, PaintEventArgs e)
        {
            var rect = new Rectangle(0, 0, pnlForPb.Width - 1, pnlForPb.Height - 1);
            e.Graphics.DrawRectangle(new Pen(Color.FromArgb(221, 221, 221)), rect);
        }
    }
}
