﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Windows.Forms;

namespace Spread.Common.Features.Utilities
{
    public static class Extensions
    {
        public static List<int> AllIndexesOf(this string str, string value)
        {
            if (String.IsNullOrEmpty(value))
                return new List<int>();
            List<int> indexes = new List<int>();
            for (int index = 0; ; index += value.Length)
            {
                index = str.IndexOf(value, index);
                if (index == -1)
                    return indexes;
                indexes.Add(index);
            }
        }

        public static T GetNext<T>(this IEnumerable<T> list, T current)
        {
            try
            {
                if (current == null || current.Equals(list.LastOrDefault()))
                    return list.FirstOrDefault();
                return list.SkipWhile(x => !x.Equals(current)).Skip(1).First();
            }
            catch
            {
                return default(T);
            }
        }

        public static T GetPrevious<T>(this IEnumerable<T> list, T current)
        {
            try
            {
                if (current == null  || current.Equals(list.FirstOrDefault()))
                    return list.LastOrDefault();
                return list.TakeWhile(x => !x.Equals(current)).Last();
            }
            catch
            {
                return default(T);
            }
        }

        public static string FromRTFToStrnig(this string rtf)
        {
            try
            {
                return new RichTextBox()
                {
                    Rtf = rtf
                }.Text;
            }
            catch
            {
                return string.Empty;
            }
            
        }

        public static string NormalizeRtf(this string source)
        {

            var result = (@"{\rtf1\ansi\deff0{\colortbl;\red100\green100\blue100;\red255\green0\blue0;\red0\green0\blue255;}" + source + " }")
                .Replace("[b]", "\\b ")
                .Replace("[/b]", "\\b0 ")
                .Replace("[color]", "\\cf1 ")
                .Replace("[/color]", "\\cf0 ")
                .Replace("\r\n", "\\line ")
                .Replace("\n", "\\line ");
            return result;
        }

        public static void FormatRfToCS(this RichTextBox rtb)
        {
            string[] reservedWords = new string[] { "abstract", "as", "base",
                "bool", "break", "byte", "case", "catch", "char", "checked",
                "class", "const", "continue", "decimal", "default", "delegate",
                "do", "double", "dynamic", "else", "enum", "event", "explicit", "extern",
                "false", "finally", "fixed", "float", "for", "foreach", "goto",
                "if", "implicit", "in", "int", "interface", "internal", "is",
                "lock", "long", "namespace", "new", "null", "object", "operator",
                "out", "override", "params","partial", "private", "protected", "public",
                "readonly", "ref", "return", "sbyte", "sealed", "short", "sizeof",
                "stackalloc", "static", "string", "struct", "switch", "this", "throw",
                "true", "try", "typeof", "uint", "ulong", "unchecked", "unsafe",
                "ushort", "using", "using static", "var", "virtual", "void", "volatile",
                "while" };

            foreach(var reservedWord in reservedWords)
            {
                foreach(int foundAt in rtb.Text.AllIndexesOf(reservedWord))
                {
                    bool validBegin = false;
                    bool validEnd = false;
                    if (foundAt > 0)
                    {
                        var beforeString = rtb.Text[foundAt - 1].ToString();
                        if (!Regex.IsMatch(beforeString, @"^[a-zA-Z0-9_]+$"))
                        {
                            validBegin = true;
                        }
                    }
                    else
                    {
                        validBegin = true;
                    }

                    if (foundAt + reservedWord.Length < rtb.Text.Length)
                    {
                        var afterString = rtb.Text[foundAt + reservedWord.Length].ToString();
                        if (!Regex.IsMatch(afterString, @"^[a-zA-Z0-9_]+$"))
                        {
                            validEnd = true;
                        }
                    }
                    else
                    {
                        validEnd = true;
                    }
                    if (validEnd && validBegin)
                    {
                        rtb.Select(foundAt, reservedWord.Length);
                        rtb.SelectionColor = System.Drawing.Color.Blue;
                    }
                }
            }

            // Find all comments and highlight them

            string commentPattern = @"((?:\/\*(?:[^*]|(?:\*+[^*\/]))*\*+\/)|(?:\/\/.*))";
            foreach(Match match in Regex.Matches(rtb.Text, commentPattern))
            {
                rtb.Select(match.Index, match.Length);
                rtb.SelectionColor = System.Drawing.Color.DarkGreen;
            }
            rtb.Select(0, 0);
            rtb.ForeColor = System.Drawing.Color.Black;
            rtb.BackColor = System.Drawing.Color.White;
        }

        [System.Runtime.InteropServices.DllImport("user32.dll")]
        public static extern int SendMessage(IntPtr hWnd, Int32 wMsg, bool wParam, Int32 lParam);

        private const int WM_SETREDRAW = 11;

        public static void SuspendDrawing(this System.Windows.Forms.Control control)
        {
            SendMessage(control.Handle, WM_SETREDRAW, false, 0);
        }

        public static void ResumeDrawing(this System.Windows.Forms.Control control)
        {
            SendMessage(control.Handle, WM_SETREDRAW, true, 0);
            control.Refresh();
        }
    }
}
