﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Spread.Common.Features.Sample
{
    public partial class BaseForm : Form
    {
        public string Title
        { get; set; }

        public string NameSpace
        { get; set; }

        public List<Type> Samples
        { get; private set; }
        public BaseForm()
        {
            InitializeComponent();
            Samples = new List<Type>();
            this.Load += OnLoad;
            WindowState = FormWindowState.Maximized;
        }

        private void OnLoad(object sender, EventArgs e)
        {
            if (DesignMode)
                return;
            InitializeDemos();
            treeView.SelectionChanged += TreeView_SelectionChanged;
            treeView.SelectedNode = treeView.Nodes[0];
        }

        private void InitializeDemos()
        {
            if(Samples.Count==0)
            {
                string nspace = string.IsNullOrEmpty(NameSpace) ? this.GetType().Namespace : NameSpace;
                Samples.AddRange(Assembly.GetEntryAssembly().GetTypes()
                    .Where(t => t.IsSubclassOf(typeof(BaseSampleControl)) && t.Namespace == nspace)
                    .OrderBy(t=>t.Name).ToList());
            }
            Samples.ForEach(t =>
            {
                var name = string.Empty;
                using (var obj = Activator.CreateInstance(t) as BaseSampleControl)
                {
                    name = obj != null ? obj.Name : t.Name.Replace("Demo", "");
                }
                var node = treeView.Nodes.Add(name);
                node.Tag = t;
                node.ShowImage = false;
            });
        }

        private void TreeView_SelectionChanged(object sender, ControlExplorer.Components.StackNodeEventArgs e)
        {
            var type = e.Node.Tag as Type;
            var obj = Activator.CreateInstance(type) as Control;
            obj.Dock = DockStyle.Fill;
            if (obj != null)
            {
                pnlDemos.Controls.Clear();
                pnlDemos.Controls.Add(obj);
                if (string.IsNullOrEmpty(Title))
                    this.Text = e.Node.Text;
                else
                    this.Text = string.Format("{0} : {1}", Title, e.Node.Text);
            }
        }
    }
}
