﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Linq;

namespace Spread.Common.Features.Sample
{
    public class Manager
    {
        public string Header { get; set; }
        public string DownloadLink { get; set; }
        public List<BaseSample> Samples = new List<BaseSample>();

        public Manager(string path)
        {
            try
            {
                XDocument xdoc = XDocument.Load(path);

                // Header
                if(xdoc.Root.Elements().Any(e => e.Name == "header"))
                {
                    var headerElement = xdoc.Root.Element("header");

                    // Title
                    if (headerElement.Elements().Any(e => e.Name == "title"))
                    {
                        var titleElement = headerElement.Element("title");
                        Header = titleElement.Value;
                    }
                    else
                    {
                        throw new Exception("`header>title` element is missing");
                    }

                    // Download Link
                    if (headerElement.Elements().Any(e => e.Name == "download"))
                    {
                        var downloadElement = headerElement.Element("download");
                        DownloadLink = downloadElement.Value;
                    }
                    else
                    {
                        throw new Exception("`header>download` element is missing");
                    }

                    // Home
                    if (headerElement.Elements().Any(e => e.Name == "home"))
                    {
                        var homeElement = headerElement.Element("home");
                        var sample = new BaseSample()
                        {
                            Title = "Home",
                            ClassName = homeElement.Attributes().Any(h => h.Name == "host") ? homeElement.Attribute("host").Value : null,
                            NotASample = true
                        };
                        Samples.Add(sample);
                    }
                    else
                    {
                        throw new Exception("`header>home` element is missing");
                    }

                    // Quickstart
                    if (headerElement.Elements().Any(e => e.Name == "quick-start"))
                    {
                        var downElement = headerElement.Element("quick-start");

                        var sample = new BaseSample()
                        {
                            Title = "Quickstart",
                            ClassName = downElement.Attributes().Any(h => h.Name == "host") ? downElement.Attribute("host").Value : null,
                            NotASample = true
                        };
                        Samples.Add(sample);
                    }
                    else
                    {
                        throw new Exception("`header>quick-start` element is missing");
                    }
                }
                else
                {
                    throw new Exception("`header` element is missing");
                }

                // Examples
                if (xdoc.Root.Elements().Any(e => e.Name == "examples"))
                {
                    var exElement = xdoc.Root.Element("examples");
                    foreach (var example in exElement.Elements().Where(elem => elem.Name == "example"))
                    {
                        try
                        {
                            var desc = example.Elements().Any(e => e.Name == "description") ? example.Element("description").Value : null;
                            var _path = example.Elements().Any(e => e.Name == "description") ?
                                example.Element("description").Attributes().Any(p => p.Name == "path") ?
                                example.Element("description").Attribute("path").Value
                                : null
                                : null;
                            if (_path != null)
                            {
                                try
                                {
                                    _path = System.IO.Path.Combine("Descriptions\\Examples", _path);
                                    desc = System.IO.File.ReadAllText(_path);
                                }
                                catch { }
                            }
                            var code = example.Elements().Any(e => e.Name == "code") ? example.Element("code").Value : null;
                            _path = example.Elements().Any(e => e.Name == "code") ?
                                example.Element("code").Attributes().Any(p => p.Name == "path") ?
                                example.Element("code").Attribute("path").Value
                                : null
                                : null;
                            if (_path != null)
                            {
                                try
                                {
                                    _path = System.IO.Path.Combine(@"Codes\Examples", _path);
                                    code = System.IO.File.ReadAllText(_path);
                                }
                                catch { }
                            }
                            var sample = new ExampleSample()
                            {
                                ExeName = example.Attributes().Any(e => e.Name == "exename") ? example.Attribute("exename").Value : null,
                                ClassName = example.Attributes().Any(e => e.Name == "className") ? example.Attribute("className").Value : null,
                                Title = example.Attributes().Any(e => e.Name == "title") ? example.Attribute("title").Value : null,
                                Status = example.Attributes().Any(e => e.Name == "status") ? example.Attribute("status").Value : null,
                                Description = desc,
                                Code = code
                            };

                            Samples.Add(sample);
                        }
                        catch
                        {
                            throw new Exception("Error parsing `example` element");
                        }
                    }
                }
                else
                {
                    throw new Exception("`examples` element is missing");
                }

                // Features
                if (xdoc.Root.Elements().Any(e => e.Name == "features"))
                {
                    var featuresElement = xdoc.Root.Element("features");                    
                    foreach(var feature in featuresElement.Elements().Where(elem => elem.Name == "feature"))
                    {
                        try
                        {
                            var desc = feature.Elements().Any(e => e.Name == "description") ? feature.Element("description").Value : null;

                            var _path = feature.Elements().Any(e => e.Name == "description") ?
                                feature.Element("description").Attributes().Any(p => p.Name == "path") ?
                                feature.Element("description").Attribute("path").Value
                                : null
                                : null;
                            if (_path != null)
                            {
                                try
                                {
                                    _path = System.IO.Path.Combine("Descriptions\\Features", _path);
                                    desc = System.IO.File.ReadAllText(_path);
                                }
                                catch { }
                            }
                            var code = feature.Elements().Any(e => e.Name == "code") ? feature.Element("code").Value : null;
                            _path = feature.Elements().Any(e => e.Name == "code") ?
                                feature.Element("code").Attributes().Any(p => p.Name == "path") ?
                                feature.Element("code").Attribute("path").Value
                                : null
                                : null;
                            if (_path != null)
                            {
                                try
                                {
                                    _path = System.IO.Path.Combine(@"Codes\Features", _path);
                                    code = System.IO.File.ReadAllText(_path);
                                }
                                catch { }
                            }

                            var sample = new FeatureSample()
                            {
                                ExeName = feature.Attributes().Any(e => e.Name == "exename") ? feature.Attribute("exename").Value : null,
                                ClassName = feature.Attributes().Any(e => e.Name == "className") ? feature.Attribute("className").Value : null,
                                Description = desc,
                                Code = code,
                                Title = feature.Attributes().Any(e => e.Name == "title") ? feature.Attribute("title").Value : null,
                                FeatureName = feature.Attributes().Any(e => e.Name == "featurename") ? feature.Attribute("featurename").Value : null,
                                Status = feature.Attributes().Any(e => e.Name == "status") ? feature.Attribute("status").Value : null,
                            };

                            Samples.Add(sample);
                        }
                        catch
                        {
                            throw new Exception("Error parsing `feature` element");
                        }
                    }
                }
                else
                {
                    throw new Exception("`features` element is missing");
                }
            }
            catch(Exception e)
            {
                System.Windows.Forms.MessageBox.Show("Config.xml's structure is invalid : " + e.Message);
            }
        }


    }
}
